@@ -119,7 +119,7 @@ static void ncci_debug(struct FsmInst *fi, char *fmt, ...)
 	if (!ncci->ncci_m.debug)
 		return;
 	va_start(args, fmt);
-	p += sprintf(p, "NCCI %08x: ", ncci->ncci);
+	p += sprintf(p, "NCCI %06x: ", ncci->ncci);
 	p += vsprintf(p, fmt, args);
 	*p = 0;
 	dprint(MIDEBUG_STATES, "%s\n", tmp);
@@ -277,7 +277,7 @@ static void ncci_facility_req(struct FsmInst *fi, int event, void *arg)
 	} else
 		mc->cmsg.Info = CapiIllMessageParmCoding;
 
-	dprintf(MIDEBUG_NCCI, "NCCI %06x: fac\n", ncci->ncci);
+	dprint(MIDEBUG_NCCI, "NCCI %06x: fac\n", ncci->ncci);
 	Send2Application(ncci, mc);
 }
 
@@ -538,6 +538,7 @@ struct mNCCI *ncciCreate(struct lPLCI *lp)
 	lp->NcciCnt++;
 	nc->ncci = lp->plci;
 	nc->ncci |= (lp->NcciCnt << 16) & 0xFFFF0000;
+	dprint(MIDEBUG_NCCI, "NCCI %06x: will be created now\n", nc->ncci);
 	nc->lp = lp;
 	nc->appl = lp->lc->Appl;
 	nc->BIlink = lp->BIlink;
@@ -571,7 +572,7 @@ struct mNCCI *ncciCreate(struct lPLCI *lp)
 			nc->flowmode = flmPHDATA;
 		}
 	} else {
-		nc->l2trans = 1;
+		nc->l2trans = 0;
 		if (lp->Bprotocol.B1 == 0) { // HDLC
 			nc->l1direct = 1;
 			nc->flowmode = flmPHDATA;
@@ -598,7 +599,7 @@ struct mNCCI *ncciCreate(struct lPLCI *lp)
 		old->next = nc;
 	} else
 		lp->Nccis = nc;
-	dprintf(MIDEBUG_NCCI, "NCCI %06x: created\n", nc->ncci);
+	dprint(MIDEBUG_NCCI, "NCCI %06x: created\n", nc->ncci);
 	return nc;
 }
 
@@ -606,7 +607,7 @@ void ncciFree(struct mNCCI *ncci)
 {
 	int i;
 
-	dprintf(MIDEBUG_NCCI, "NCCI %06x: free\n", ncci->ncci);
+	dprint(MIDEBUG_NCCI, "NCCI %06x: free\n", ncci->ncci);
 
 	/* cleanup data queues */
 	for (i = 0; i < CAPI_MAXDATAWINDOW; i++) {
@@ -617,13 +618,13 @@ void ncciFree(struct mNCCI *ncci)
 		lPLCIDelNCCI(ncci);
 	else
 		wprint("NCCI %06x: PLCI not linked\n", ncci->ncci);
-	dprintf(MIDEBUG_NCCI, "NCCI %06x: freed\n", ncci->ncci);
+	dprint(MIDEBUG_NCCI, "NCCI %06x: freed\n", ncci->ncci);
 	free(ncci);
 }
 
 void ncciDel_lPlci(struct mNCCI *ncci)
 {
-	dprintf(MIDEBUG_NCCI, "NCCI %06x: unlink PLCI:%04x\n", ncci->ncci, ncci->lp ? ncci->lp->plci : -1);
+	dprint(MIDEBUG_NCCI, "NCCI %06x: unlink PLCI:%04x\n", ncci->ncci, ncci->lp ? ncci->lp->plci : -1);
 	ncci->lp = NULL;
 	/* maybe we should release the NCCI here */
 }
@@ -1139,17 +1140,20 @@ static int ncciSendMessage(struct mNCCI *ncci, uint8_t cmd, uint8_t subcmd, stru
 
 int ncciB3Data(struct BInstance *bi, struct mc_buf *mc)
 {
-	struct mNCCI *ncci = bi->b3data;
+	struct mNCCI *ncci;
 
 	if (mc->cmsg.Command == CAPI_CONNECT_B3 && mc->cmsg.Subcommand == CAPI_REQ) {
+		pthread_mutex_lock(&bi->lp->lock);
+		ncci = bi->b3data;
 		if (ncci)
 			wprint("NCCI %06x: already assigned\n", ncci->ncci);
 		else {
 			ncci = ConnectB3Request(bi->lp, mc);
 			bi->b3data = ncci;
 		}
-			
-	}
+		pthread_mutex_unlock(&bi->lp->lock);	
+	} else
+		ncci = bi->b3data;
 	if (!ncci) {
 		wprint("No NCCI asigned for  PCLI %04x\n", bi->lp->plci);
 		return -EINVAL;
@@ -1291,12 +1295,15 @@ int recvBdirect(struct BInstance *bi, struct mc_buf *mc)
 	case PH_ACTIVATE_CNF:
 	case DL_ESTABLISH_CNF:
 		if (!ncci) {
+			pthread_mutex_lock(&bi->lp->lock);
 			ncci = ncciCreate(bi->lp);
 			if (!ncci) {
+				pthread_mutex_unlock(&bi->lp->lock);
 				eprint("Cannot create NCCI for PLCI %04x\n", bi->lp ? bi->lp->plci : 0xffff);
 				return -ENOMEM;
 			} else
 				bi->b3data = ncci;
+			pthread_mutex_unlock(&bi->lp->lock);
 		}
 		FsmEvent(&ncci->ncci_m, EV_DL_ESTABLISH_CONF, mc);
 #ifdef UNSINN
@@ -1312,12 +1319,15 @@ int recvBdirect(struct BInstance *bi, struct mc_buf *mc)
 	case PH_ACTIVATE_IND:
 	case DL_ESTABLISH_IND:
 		if (!ncci) {
+			pthread_mutex_lock(&bi->lp->lock);
 			ncci = ncciCreate(bi->lp);
 			if (!ncci) {
+				pthread_mutex_unlock(&bi->lp->lock);
 				eprint("Cannot create NCCI for PLCI %04x\n", bi->lp ? bi->lp->plci : 0xffff);
 				return -ENOMEM;
 			} else
 				bi->b3data = ncci;
+			pthread_mutex_unlock(&bi->lp->lock);
 		} else
 			dprint(MIDEBUG_NCCI, "NCCI %06x: %s on existing NCCIx\n", ncci->ncci, _mi_msg_type2str(hh->prim));
 		FsmEvent(&ncci->ncci_m, EV_DL_ESTABLISH_IND, mc);
